
//you need the video library
import processing.video.*;

Movie video;

//the buffer for storing video frames
ArrayList frames;

//different program modes for recording and playback
int mode = 0;
int MODE_buffer = 0;
int MODE_recordPlay = 1; 
//recording now goes directly into playback, no separate mode necessary
//int MODE_PLAYBACK = 2;

//variable storing the x value of the video slice being displayed over time
int currentX = 0;

void setup() {
  //NOTE IF USING STEREO VIDEO!!!
  //because the stereo video must be processed with a horizontal slit, the video
  //is just pre-rotated ninety degress.
  //in the future the sketch may be rewritten to not require pre-rotating the video
  //as it is though, this sketch works properly with non-stereo vids!
  size(320, 240);
  
  // video input to get your video file
  video = new Movie(this, "samplevid.mp4");
  
  // Start the video in a loop
  video.loop();  
}
//movie event is called outside of draw
void movieEvent(Movie c) {
  c.read();

  //begin the initial buffer of frames then send to simultaneous recording/playback 
  if (mode == MODE_buffer) {
    frames = new ArrayList();
    mode = MODE_recordPlay;
  }

  //record into the buffer until there are enough frames
  //sketch will be BLANK until the array size = the sketch width
  if (mode == MODE_recordPlay) {
    //copy the current video frame into an image, so it can be stored in the buffer
    PImage img = createImage(width, height, RGB);
    video.loadPixels();
    arrayCopy(video.pixels, img.pixels);
    //adding video frames as images to the array
    frames.add(img);

  }
}

void draw() { 
  loadPixels();

  //code for the recording mode 
  if (mode == MODE_recordPlay) {
    //set the image counter to 0
    int currentImage = 0;

    //begin a loop for the pixel columns from the video
    for (int x = 0; x < video.width; x++) {
      //go through the frame buffer and pick an image using the image counter
      if (currentImage < frames.size()) {
        //increase the image counter
        currentImage++;
        //just print the size of the array
        println(frames.size());
        // we only need to keep enough frames in the array to fill the window, so when the array
        //is larger than the window width + 1, the oldest frame is removed 
        if (frames.size() >= width+1) {
          frames.remove(0);
          for (int z = 0; z < video.width; z++) {
      //get an image from the buffer using loopcounter z as the index
      PImage img = (PImage)frames.get(z);

      if (img != null) {
        img.loadPixels();
        
        //pick the same column from each image for display, 
        //then distribute the columns over the x-axis on the screen
        //does this by writing to the pixel array
        //this has the effect of reversing the dimensions of X and Time 
        //(X dimension is here represented by variable z because x has already been used)
        for(int y = 0; y < video.height; y++) {
          pixels[z + y * width] = img.pixels[currentX + y * video.width];
        }  
      }
    } 
    currentX++;
        if(currentX >= video.width) {
      //reset the column counter
      //do not create a new buffer
      currentX = 0;
    }
        }
      } 
      else {
        break;
      }
    }
  }

  //updating the pixel array
  updatePixels();
}